
import google.protobuf
from packaging.version import Version
if Version(google.protobuf.__version__).major >= 5:
  # -*- coding: utf-8 -*-
  # Generated by the protocol buffer compiler.  DO NOT EDIT!
  # source: databricks_artifacts.proto
  # Protobuf Python Version: 5.26.0
  """Generated protocol buffer code."""
  from google.protobuf import descriptor as _descriptor
  from google.protobuf import descriptor_pool as _descriptor_pool
  from google.protobuf import symbol_database as _symbol_database
  from google.protobuf.internal import builder as _builder
  # @@protoc_insertion_point(imports)

  _sym_db = _symbol_database.Default()


  from .scalapb import scalapb_pb2 as scalapb_dot_scalapb__pb2
  from . import databricks_pb2 as databricks__pb2


  DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1a\x64\x61tabricks_artifacts.proto\x12\x06mlflow\x1a\x15scalapb/scalapb.proto\x1a\x10\x64\x61tabricks.proto\"\xdf\x01\n\x16\x41rtifactCredentialInfo\x12\x0e\n\x06run_id\x18\x01 \x01(\t\x12\x0c\n\x04path\x18\x02 \x01(\t\x12\x12\n\nsigned_uri\x18\x03 \x01(\t\x12:\n\x07headers\x18\x04 \x03(\x0b\x32).mlflow.ArtifactCredentialInfo.HttpHeader\x12,\n\x04type\x18\x05 \x01(\x0e\x32\x1e.mlflow.ArtifactCredentialType\x1a)\n\nHttpHeader\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t\"j\n\x1dLoggedModelArtifactCredential\x12\x10\n\x08model_id\x18\x01 \x01(\t\x12\x37\n\x0f\x63redential_info\x18\x02 \x01(\x0b\x32\x1e.mlflow.ArtifactCredentialInfo\"\xe1\x01\n\x15GetCredentialsForRead\x12\x14\n\x06run_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x0c\n\x04path\x18\x02 \x03(\t\x12\x12\n\npage_token\x18\x03 \x01(\t\x1a\x63\n\x08Response\x12\x38\n\x10\x63redential_infos\x18\x02 \x03(\x0b\x32\x1e.mlflow.ArtifactCredentialInfo\x12\x17\n\x0fnext_page_token\x18\x03 \x01(\tJ\x04\x08\x01\x10\x02:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xe2\x01\n\x16GetCredentialsForWrite\x12\x14\n\x06run_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x0c\n\x04path\x18\x02 \x03(\t\x12\x12\n\npage_token\x18\x03 \x01(\t\x1a\x63\n\x08Response\x12\x38\n\x10\x63redential_infos\x18\x02 \x03(\x0b\x32\x1e.mlflow.ArtifactCredentialInfo\x12\x17\n\x0fnext_page_token\x18\x03 \x01(\tJ\x04\x08\x01\x10\x02:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xa1\x02\n\x15\x43reateMultipartUpload\x12\x14\n\x06run_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x0c\n\x04path\x18\x02 \x01(\t\x12\x17\n\tnum_parts\x18\x03 \x01(\x03\x42\x04\xf8\x86\x19\x01\x1a\x9d\x01\n\x08Response\x12\x11\n\tupload_id\x18\x01 \x01(\t\x12?\n\x17upload_credential_infos\x18\x02 \x03(\x0b\x32\x1e.mlflow.ArtifactCredentialInfo\x12=\n\x15\x61\x62ort_credential_info\x18\x03 \x01(\x0b\x32\x1e.mlflow.ArtifactCredentialInfo:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"-\n\x08PartEtag\x12\x13\n\x0bpart_number\x18\x01 \x01(\x03\x12\x0c\n\x04\x65tag\x18\x02 \x01(\t\"\xb5\x01\n\x17\x43ompleteMultipartUpload\x12\x14\n\x06run_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x0c\n\x04path\x18\x02 \x01(\t\x12\x17\n\tupload_id\x18\x03 \x01(\tB\x04\xf8\x86\x19\x01\x12$\n\npart_etags\x18\x04 \x03(\x0b\x32\x10.mlflow.PartEtag\x1a\n\n\x08Response:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xec\x01\n\x19GetPresignedUploadPartUrl\x12\x14\n\x06run_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x0c\n\x04path\x18\x02 \x01(\t\x12\x17\n\tupload_id\x18\x03 \x01(\tB\x04\xf8\x86\x19\x01\x12\x19\n\x0bpart_number\x18\x04 \x01(\x03\x42\x04\xf8\x86\x19\x01\x1aJ\n\x08Response\x12>\n\x16upload_credential_info\x18\x01 \x01(\x0b\x32\x1e.mlflow.ArtifactCredentialInfo:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xb0\x01\n\"GetCredentialsForTraceDataDownload\x12\x18\n\nrequest_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x1a\x43\n\x08Response\x12\x37\n\x0f\x63redential_info\x18\x01 \x01(\x0b\x32\x1e.mlflow.ArtifactCredentialInfo:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xae\x01\n GetCredentialsForTraceDataUpload\x12\x18\n\nrequest_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x1a\x43\n\x08Response\x12\x37\n\x0f\x63redential_info\x18\x01 \x01(\x0b\x32\x1e.mlflow.ArtifactCredentialInfo:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xed\x01\n\"GetCredentialsForLoggedModelUpload\x12\x16\n\x08model_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\r\n\x05paths\x18\x02 \x03(\t\x12\x12\n\npage_token\x18\x03 \x01(\t\x1a_\n\x08Response\x12:\n\x0b\x63redentials\x18\x01 \x03(\x0b\x32%.mlflow.LoggedModelArtifactCredential\x12\x17\n\x0fnext_page_token\x18\x02 \x01(\t:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xef\x01\n$GetCredentialsForLoggedModelDownload\x12\x16\n\x08model_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\r\n\x05paths\x18\x02 \x03(\t\x12\x12\n\npage_token\x18\x03 \x01(\t\x1a_\n\x08Response\x12:\n\x0b\x63redentials\x18\x01 \x03(\x0b\x32%.mlflow.LoggedModelArtifactCredential\x12\x17\n\x0fnext_page_token\x18\x02 \x01(\t:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]*s\n\x16\x41rtifactCredentialType\x12\x11\n\rAZURE_SAS_URI\x10\x01\x12\x15\n\x11\x41WS_PRESIGNED_URL\x10\x02\x12\x12\n\x0eGCP_SIGNED_URL\x10\x03\x12\x1b\n\x17\x41ZURE_ADLS_GEN2_SAS_URI\x10\x04\x32\xd5\r\n DatabricksMlflowArtifactsService\x12\x9c\x01\n\x15getCredentialsForRead\x12\x1d.mlflow.GetCredentialsForRead\x1a&.mlflow.GetCredentialsForRead.Response\"<\xf2\x86\x19\x38\n4\n\x04POST\x12&/mlflow/artifacts/credentials-for-read\x1a\x04\x08\x02\x10\x00\x10\x03\x12\xa0\x01\n\x16getCredentialsForWrite\x12\x1e.mlflow.GetCredentialsForWrite\x1a\'.mlflow.GetCredentialsForWrite.Response\"=\xf2\x86\x19\x39\n5\n\x04POST\x12\'/mlflow/artifacts/credentials-for-write\x1a\x04\x08\x02\x10\x00\x10\x03\x12\x9f\x01\n\x15\x63reateMultipartUpload\x12\x1d.mlflow.CreateMultipartUpload\x1a&.mlflow.CreateMultipartUpload.Response\"?\xf2\x86\x19;\n7\n\x04POST\x12)/mlflow/artifacts/create-multipart-upload\x1a\x04\x08\x02\x10\x00\x10\x03\x12\xa7\x01\n\x17\x63ompleteMultipartUpload\x12\x1f.mlflow.CompleteMultipartUpload\x1a(.mlflow.CompleteMultipartUpload.Response\"A\xf2\x86\x19=\n9\n\x04POST\x12+/mlflow/artifacts/complete-multipart-upload\x1a\x04\x08\x02\x10\x00\x10\x03\x12\xb0\x01\n\x19getPresignedUploadPartUrl\x12!.mlflow.GetPresignedUploadPartUrl\x1a*.mlflow.GetPresignedUploadPartUrl.Response\"D\xf2\x86\x19@\n<\n\x03GET\x12//mlflow/artifacts/get-presigned-upload-part-url\x1a\x04\x08\x02\x10\x00\x10\x03\x12\xd5\x01\n\"getCredentialsForTraceDataDownload\x12*.mlflow.GetCredentialsForTraceDataDownload\x1a\x33.mlflow.GetCredentialsForTraceDataDownload.Response\"N\xf2\x86\x19J\nF\n\x03GET\x12\x39/mlflow/traces/{request_id}/credentials-for-data-download\x1a\x04\x08\x02\x10\x00\x10\x03\x12\xcd\x01\n getCredentialsForTraceDataUpload\x12(.mlflow.GetCredentialsForTraceDataUpload\x1a\x31.mlflow.GetCredentialsForTraceDataUpload.Response\"L\xf2\x86\x19H\nD\n\x03GET\x12\x37/mlflow/traces/{request_id}/credentials-for-data-upload\x1a\x04\x08\x02\x10\x00\x10\x03\x12\xde\x01\n\"getCredentialsForLoggedModelUpload\x12*.mlflow.GetCredentialsForLoggedModelUpload\x1a\x33.mlflow.GetCredentialsForLoggedModelUpload.Response\"W\xf2\x86\x19S\nO\n\x04POST\x12\x41/mlflow/logged-models/{model_id}/artifacts/credentials-for-upload\x1a\x04\x08\x02\x10\x00\x10\x03\x12\xe6\x01\n$getCredentialsForLoggedModelDownload\x12,.mlflow.GetCredentialsForLoggedModelDownload\x1a\x35.mlflow.GetCredentialsForLoggedModelDownload.Response\"Y\xf2\x86\x19U\nQ\n\x04POST\x12\x43/mlflow/logged-models/{model_id}/artifacts/credentials-for-download\x1a\x04\x08\x02\x10\x00\x10\x03\x42,\n\x1f\x63om.databricks.api.proto.mlflow\x90\x01\x01\xa0\x01\x01\xe2?\x02\x10\x01')

  _globals = globals()
  _builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
  _builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'databricks_artifacts_pb2', _globals)
  if not _descriptor._USE_C_DESCRIPTORS:
    _globals['DESCRIPTOR']._loaded_options = None
    _globals['DESCRIPTOR']._serialized_options = b'\n\037com.databricks.api.proto.mlflow\220\001\001\240\001\001\342?\002\020\001'
    _globals['_GETCREDENTIALSFORREAD'].fields_by_name['run_id']._loaded_options = None
    _globals['_GETCREDENTIALSFORREAD'].fields_by_name['run_id']._serialized_options = b'\370\206\031\001'
    _globals['_GETCREDENTIALSFORREAD']._loaded_options = None
    _globals['_GETCREDENTIALSFORREAD']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_GETCREDENTIALSFORWRITE'].fields_by_name['run_id']._loaded_options = None
    _globals['_GETCREDENTIALSFORWRITE'].fields_by_name['run_id']._serialized_options = b'\370\206\031\001'
    _globals['_GETCREDENTIALSFORWRITE']._loaded_options = None
    _globals['_GETCREDENTIALSFORWRITE']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_CREATEMULTIPARTUPLOAD'].fields_by_name['run_id']._loaded_options = None
    _globals['_CREATEMULTIPARTUPLOAD'].fields_by_name['run_id']._serialized_options = b'\370\206\031\001'
    _globals['_CREATEMULTIPARTUPLOAD'].fields_by_name['num_parts']._loaded_options = None
    _globals['_CREATEMULTIPARTUPLOAD'].fields_by_name['num_parts']._serialized_options = b'\370\206\031\001'
    _globals['_CREATEMULTIPARTUPLOAD']._loaded_options = None
    _globals['_CREATEMULTIPARTUPLOAD']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_COMPLETEMULTIPARTUPLOAD'].fields_by_name['run_id']._loaded_options = None
    _globals['_COMPLETEMULTIPARTUPLOAD'].fields_by_name['run_id']._serialized_options = b'\370\206\031\001'
    _globals['_COMPLETEMULTIPARTUPLOAD'].fields_by_name['upload_id']._loaded_options = None
    _globals['_COMPLETEMULTIPARTUPLOAD'].fields_by_name['upload_id']._serialized_options = b'\370\206\031\001'
    _globals['_COMPLETEMULTIPARTUPLOAD']._loaded_options = None
    _globals['_COMPLETEMULTIPARTUPLOAD']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_GETPRESIGNEDUPLOADPARTURL'].fields_by_name['run_id']._loaded_options = None
    _globals['_GETPRESIGNEDUPLOADPARTURL'].fields_by_name['run_id']._serialized_options = b'\370\206\031\001'
    _globals['_GETPRESIGNEDUPLOADPARTURL'].fields_by_name['upload_id']._loaded_options = None
    _globals['_GETPRESIGNEDUPLOADPARTURL'].fields_by_name['upload_id']._serialized_options = b'\370\206\031\001'
    _globals['_GETPRESIGNEDUPLOADPARTURL'].fields_by_name['part_number']._loaded_options = None
    _globals['_GETPRESIGNEDUPLOADPARTURL'].fields_by_name['part_number']._serialized_options = b'\370\206\031\001'
    _globals['_GETPRESIGNEDUPLOADPARTURL']._loaded_options = None
    _globals['_GETPRESIGNEDUPLOADPARTURL']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_GETCREDENTIALSFORTRACEDATADOWNLOAD'].fields_by_name['request_id']._loaded_options = None
    _globals['_GETCREDENTIALSFORTRACEDATADOWNLOAD'].fields_by_name['request_id']._serialized_options = b'\370\206\031\001'
    _globals['_GETCREDENTIALSFORTRACEDATADOWNLOAD']._loaded_options = None
    _globals['_GETCREDENTIALSFORTRACEDATADOWNLOAD']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_GETCREDENTIALSFORTRACEDATAUPLOAD'].fields_by_name['request_id']._loaded_options = None
    _globals['_GETCREDENTIALSFORTRACEDATAUPLOAD'].fields_by_name['request_id']._serialized_options = b'\370\206\031\001'
    _globals['_GETCREDENTIALSFORTRACEDATAUPLOAD']._loaded_options = None
    _globals['_GETCREDENTIALSFORTRACEDATAUPLOAD']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_GETCREDENTIALSFORLOGGEDMODELUPLOAD'].fields_by_name['model_id']._loaded_options = None
    _globals['_GETCREDENTIALSFORLOGGEDMODELUPLOAD'].fields_by_name['model_id']._serialized_options = b'\370\206\031\001'
    _globals['_GETCREDENTIALSFORLOGGEDMODELUPLOAD']._loaded_options = None
    _globals['_GETCREDENTIALSFORLOGGEDMODELUPLOAD']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_GETCREDENTIALSFORLOGGEDMODELDOWNLOAD'].fields_by_name['model_id']._loaded_options = None
    _globals['_GETCREDENTIALSFORLOGGEDMODELDOWNLOAD'].fields_by_name['model_id']._serialized_options = b'\370\206\031\001'
    _globals['_GETCREDENTIALSFORLOGGEDMODELDOWNLOAD']._loaded_options = None
    _globals['_GETCREDENTIALSFORLOGGEDMODELDOWNLOAD']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_DATABRICKSMLFLOWARTIFACTSSERVICE'].methods_by_name['getCredentialsForRead']._loaded_options = None
    _globals['_DATABRICKSMLFLOWARTIFACTSSERVICE'].methods_by_name['getCredentialsForRead']._serialized_options = b'\362\206\0318\n4\n\004POST\022&/mlflow/artifacts/credentials-for-read\032\004\010\002\020\000\020\003'
    _globals['_DATABRICKSMLFLOWARTIFACTSSERVICE'].methods_by_name['getCredentialsForWrite']._loaded_options = None
    _globals['_DATABRICKSMLFLOWARTIFACTSSERVICE'].methods_by_name['getCredentialsForWrite']._serialized_options = b'\362\206\0319\n5\n\004POST\022\'/mlflow/artifacts/credentials-for-write\032\004\010\002\020\000\020\003'
    _globals['_DATABRICKSMLFLOWARTIFACTSSERVICE'].methods_by_name['createMultipartUpload']._loaded_options = None
    _globals['_DATABRICKSMLFLOWARTIFACTSSERVICE'].methods_by_name['createMultipartUpload']._serialized_options = b'\362\206\031;\n7\n\004POST\022)/mlflow/artifacts/create-multipart-upload\032\004\010\002\020\000\020\003'
    _globals['_DATABRICKSMLFLOWARTIFACTSSERVICE'].methods_by_name['completeMultipartUpload']._loaded_options = None
    _globals['_DATABRICKSMLFLOWARTIFACTSSERVICE'].methods_by_name['completeMultipartUpload']._serialized_options = b'\362\206\031=\n9\n\004POST\022+/mlflow/artifacts/complete-multipart-upload\032\004\010\002\020\000\020\003'
    _globals['_DATABRICKSMLFLOWARTIFACTSSERVICE'].methods_by_name['getPresignedUploadPartUrl']._loaded_options = None
    _globals['_DATABRICKSMLFLOWARTIFACTSSERVICE'].methods_by_name['getPresignedUploadPartUrl']._serialized_options = b'\362\206\031@\n<\n\003GET\022//mlflow/artifacts/get-presigned-upload-part-url\032\004\010\002\020\000\020\003'
    _globals['_DATABRICKSMLFLOWARTIFACTSSERVICE'].methods_by_name['getCredentialsForTraceDataDownload']._loaded_options = None
    _globals['_DATABRICKSMLFLOWARTIFACTSSERVICE'].methods_by_name['getCredentialsForTraceDataDownload']._serialized_options = b'\362\206\031J\nF\n\003GET\0229/mlflow/traces/{request_id}/credentials-for-data-download\032\004\010\002\020\000\020\003'
    _globals['_DATABRICKSMLFLOWARTIFACTSSERVICE'].methods_by_name['getCredentialsForTraceDataUpload']._loaded_options = None
    _globals['_DATABRICKSMLFLOWARTIFACTSSERVICE'].methods_by_name['getCredentialsForTraceDataUpload']._serialized_options = b'\362\206\031H\nD\n\003GET\0227/mlflow/traces/{request_id}/credentials-for-data-upload\032\004\010\002\020\000\020\003'
    _globals['_DATABRICKSMLFLOWARTIFACTSSERVICE'].methods_by_name['getCredentialsForLoggedModelUpload']._loaded_options = None
    _globals['_DATABRICKSMLFLOWARTIFACTSSERVICE'].methods_by_name['getCredentialsForLoggedModelUpload']._serialized_options = b'\362\206\031S\nO\n\004POST\022A/mlflow/logged-models/{model_id}/artifacts/credentials-for-upload\032\004\010\002\020\000\020\003'
    _globals['_DATABRICKSMLFLOWARTIFACTSSERVICE'].methods_by_name['getCredentialsForLoggedModelDownload']._loaded_options = None
    _globals['_DATABRICKSMLFLOWARTIFACTSSERVICE'].methods_by_name['getCredentialsForLoggedModelDownload']._serialized_options = b'\362\206\031U\nQ\n\004POST\022C/mlflow/logged-models/{model_id}/artifacts/credentials-for-download\032\004\010\002\020\000\020\003'
    _globals['_ARTIFACTCREDENTIALTYPE']._serialized_start=2470
    _globals['_ARTIFACTCREDENTIALTYPE']._serialized_end=2585
    _globals['_ARTIFACTCREDENTIALINFO']._serialized_start=80
    _globals['_ARTIFACTCREDENTIALINFO']._serialized_end=303
    _globals['_ARTIFACTCREDENTIALINFO_HTTPHEADER']._serialized_start=262
    _globals['_ARTIFACTCREDENTIALINFO_HTTPHEADER']._serialized_end=303
    _globals['_LOGGEDMODELARTIFACTCREDENTIAL']._serialized_start=305
    _globals['_LOGGEDMODELARTIFACTCREDENTIAL']._serialized_end=411
    _globals['_GETCREDENTIALSFORREAD']._serialized_start=414
    _globals['_GETCREDENTIALSFORREAD']._serialized_end=639
    _globals['_GETCREDENTIALSFORREAD_RESPONSE']._serialized_start=495
    _globals['_GETCREDENTIALSFORREAD_RESPONSE']._serialized_end=594
    _globals['_GETCREDENTIALSFORWRITE']._serialized_start=642
    _globals['_GETCREDENTIALSFORWRITE']._serialized_end=868
    _globals['_GETCREDENTIALSFORWRITE_RESPONSE']._serialized_start=495
    _globals['_GETCREDENTIALSFORWRITE_RESPONSE']._serialized_end=594
    _globals['_CREATEMULTIPARTUPLOAD']._serialized_start=871
    _globals['_CREATEMULTIPARTUPLOAD']._serialized_end=1160
    _globals['_CREATEMULTIPARTUPLOAD_RESPONSE']._serialized_start=958
    _globals['_CREATEMULTIPARTUPLOAD_RESPONSE']._serialized_end=1115
    _globals['_PARTETAG']._serialized_start=1162
    _globals['_PARTETAG']._serialized_end=1207
    _globals['_COMPLETEMULTIPARTUPLOAD']._serialized_start=1210
    _globals['_COMPLETEMULTIPARTUPLOAD']._serialized_end=1391
    _globals['_COMPLETEMULTIPARTUPLOAD_RESPONSE']._serialized_start=495
    _globals['_COMPLETEMULTIPARTUPLOAD_RESPONSE']._serialized_end=505
    _globals['_GETPRESIGNEDUPLOADPARTURL']._serialized_start=1394
    _globals['_GETPRESIGNEDUPLOADPARTURL']._serialized_end=1630
    _globals['_GETPRESIGNEDUPLOADPARTURL_RESPONSE']._serialized_start=1511
    _globals['_GETPRESIGNEDUPLOADPARTURL_RESPONSE']._serialized_end=1585
    _globals['_GETCREDENTIALSFORTRACEDATADOWNLOAD']._serialized_start=1633
    _globals['_GETCREDENTIALSFORTRACEDATADOWNLOAD']._serialized_end=1809
    _globals['_GETCREDENTIALSFORTRACEDATADOWNLOAD_RESPONSE']._serialized_start=1697
    _globals['_GETCREDENTIALSFORTRACEDATADOWNLOAD_RESPONSE']._serialized_end=1764
    _globals['_GETCREDENTIALSFORTRACEDATAUPLOAD']._serialized_start=1812
    _globals['_GETCREDENTIALSFORTRACEDATAUPLOAD']._serialized_end=1986
    _globals['_GETCREDENTIALSFORTRACEDATAUPLOAD_RESPONSE']._serialized_start=1697
    _globals['_GETCREDENTIALSFORTRACEDATAUPLOAD_RESPONSE']._serialized_end=1764
    _globals['_GETCREDENTIALSFORLOGGEDMODELUPLOAD']._serialized_start=1989
    _globals['_GETCREDENTIALSFORLOGGEDMODELUPLOAD']._serialized_end=2226
    _globals['_GETCREDENTIALSFORLOGGEDMODELUPLOAD_RESPONSE']._serialized_start=2086
    _globals['_GETCREDENTIALSFORLOGGEDMODELUPLOAD_RESPONSE']._serialized_end=2181
    _globals['_GETCREDENTIALSFORLOGGEDMODELDOWNLOAD']._serialized_start=2229
    _globals['_GETCREDENTIALSFORLOGGEDMODELDOWNLOAD']._serialized_end=2468
    _globals['_GETCREDENTIALSFORLOGGEDMODELDOWNLOAD_RESPONSE']._serialized_start=2086
    _globals['_GETCREDENTIALSFORLOGGEDMODELDOWNLOAD_RESPONSE']._serialized_end=2181
    _globals['_DATABRICKSMLFLOWARTIFACTSSERVICE']._serialized_start=2588
    _globals['_DATABRICKSMLFLOWARTIFACTSSERVICE']._serialized_end=4337
  _builder.BuildServices(DESCRIPTOR, 'databricks_artifacts_pb2', _globals)
  # @@protoc_insertion_point(module_scope)

else:
  # -*- coding: utf-8 -*-
  # Generated by the protocol buffer compiler.  DO NOT EDIT!
  # source: databricks_artifacts.proto
  """Generated protocol buffer code."""
  from google.protobuf.internal import enum_type_wrapper
  from google.protobuf import descriptor as _descriptor
  from google.protobuf import descriptor_pool as _descriptor_pool
  from google.protobuf import message as _message
  from google.protobuf import reflection as _reflection
  from google.protobuf import symbol_database as _symbol_database
  from google.protobuf import service as _service
  from google.protobuf import service_reflection
  # @@protoc_insertion_point(imports)

  _sym_db = _symbol_database.Default()


  from .scalapb import scalapb_pb2 as scalapb_dot_scalapb__pb2
  from . import databricks_pb2 as databricks__pb2


  DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1a\x64\x61tabricks_artifacts.proto\x12\x06mlflow\x1a\x15scalapb/scalapb.proto\x1a\x10\x64\x61tabricks.proto\"\xdf\x01\n\x16\x41rtifactCredentialInfo\x12\x0e\n\x06run_id\x18\x01 \x01(\t\x12\x0c\n\x04path\x18\x02 \x01(\t\x12\x12\n\nsigned_uri\x18\x03 \x01(\t\x12:\n\x07headers\x18\x04 \x03(\x0b\x32).mlflow.ArtifactCredentialInfo.HttpHeader\x12,\n\x04type\x18\x05 \x01(\x0e\x32\x1e.mlflow.ArtifactCredentialType\x1a)\n\nHttpHeader\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t\"j\n\x1dLoggedModelArtifactCredential\x12\x10\n\x08model_id\x18\x01 \x01(\t\x12\x37\n\x0f\x63redential_info\x18\x02 \x01(\x0b\x32\x1e.mlflow.ArtifactCredentialInfo\"\xe1\x01\n\x15GetCredentialsForRead\x12\x14\n\x06run_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x0c\n\x04path\x18\x02 \x03(\t\x12\x12\n\npage_token\x18\x03 \x01(\t\x1a\x63\n\x08Response\x12\x38\n\x10\x63redential_infos\x18\x02 \x03(\x0b\x32\x1e.mlflow.ArtifactCredentialInfo\x12\x17\n\x0fnext_page_token\x18\x03 \x01(\tJ\x04\x08\x01\x10\x02:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xe2\x01\n\x16GetCredentialsForWrite\x12\x14\n\x06run_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x0c\n\x04path\x18\x02 \x03(\t\x12\x12\n\npage_token\x18\x03 \x01(\t\x1a\x63\n\x08Response\x12\x38\n\x10\x63redential_infos\x18\x02 \x03(\x0b\x32\x1e.mlflow.ArtifactCredentialInfo\x12\x17\n\x0fnext_page_token\x18\x03 \x01(\tJ\x04\x08\x01\x10\x02:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xa1\x02\n\x15\x43reateMultipartUpload\x12\x14\n\x06run_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x0c\n\x04path\x18\x02 \x01(\t\x12\x17\n\tnum_parts\x18\x03 \x01(\x03\x42\x04\xf8\x86\x19\x01\x1a\x9d\x01\n\x08Response\x12\x11\n\tupload_id\x18\x01 \x01(\t\x12?\n\x17upload_credential_infos\x18\x02 \x03(\x0b\x32\x1e.mlflow.ArtifactCredentialInfo\x12=\n\x15\x61\x62ort_credential_info\x18\x03 \x01(\x0b\x32\x1e.mlflow.ArtifactCredentialInfo:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"-\n\x08PartEtag\x12\x13\n\x0bpart_number\x18\x01 \x01(\x03\x12\x0c\n\x04\x65tag\x18\x02 \x01(\t\"\xb5\x01\n\x17\x43ompleteMultipartUpload\x12\x14\n\x06run_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x0c\n\x04path\x18\x02 \x01(\t\x12\x17\n\tupload_id\x18\x03 \x01(\tB\x04\xf8\x86\x19\x01\x12$\n\npart_etags\x18\x04 \x03(\x0b\x32\x10.mlflow.PartEtag\x1a\n\n\x08Response:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xec\x01\n\x19GetPresignedUploadPartUrl\x12\x14\n\x06run_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x0c\n\x04path\x18\x02 \x01(\t\x12\x17\n\tupload_id\x18\x03 \x01(\tB\x04\xf8\x86\x19\x01\x12\x19\n\x0bpart_number\x18\x04 \x01(\x03\x42\x04\xf8\x86\x19\x01\x1aJ\n\x08Response\x12>\n\x16upload_credential_info\x18\x01 \x01(\x0b\x32\x1e.mlflow.ArtifactCredentialInfo:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xb0\x01\n\"GetCredentialsForTraceDataDownload\x12\x18\n\nrequest_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x1a\x43\n\x08Response\x12\x37\n\x0f\x63redential_info\x18\x01 \x01(\x0b\x32\x1e.mlflow.ArtifactCredentialInfo:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xae\x01\n GetCredentialsForTraceDataUpload\x12\x18\n\nrequest_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x1a\x43\n\x08Response\x12\x37\n\x0f\x63redential_info\x18\x01 \x01(\x0b\x32\x1e.mlflow.ArtifactCredentialInfo:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xed\x01\n\"GetCredentialsForLoggedModelUpload\x12\x16\n\x08model_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\r\n\x05paths\x18\x02 \x03(\t\x12\x12\n\npage_token\x18\x03 \x01(\t\x1a_\n\x08Response\x12:\n\x0b\x63redentials\x18\x01 \x03(\x0b\x32%.mlflow.LoggedModelArtifactCredential\x12\x17\n\x0fnext_page_token\x18\x02 \x01(\t:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xef\x01\n$GetCredentialsForLoggedModelDownload\x12\x16\n\x08model_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\r\n\x05paths\x18\x02 \x03(\t\x12\x12\n\npage_token\x18\x03 \x01(\t\x1a_\n\x08Response\x12:\n\x0b\x63redentials\x18\x01 \x03(\x0b\x32%.mlflow.LoggedModelArtifactCredential\x12\x17\n\x0fnext_page_token\x18\x02 \x01(\t:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]*s\n\x16\x41rtifactCredentialType\x12\x11\n\rAZURE_SAS_URI\x10\x01\x12\x15\n\x11\x41WS_PRESIGNED_URL\x10\x02\x12\x12\n\x0eGCP_SIGNED_URL\x10\x03\x12\x1b\n\x17\x41ZURE_ADLS_GEN2_SAS_URI\x10\x04\x32\xd5\r\n DatabricksMlflowArtifactsService\x12\x9c\x01\n\x15getCredentialsForRead\x12\x1d.mlflow.GetCredentialsForRead\x1a&.mlflow.GetCredentialsForRead.Response\"<\xf2\x86\x19\x38\n4\n\x04POST\x12&/mlflow/artifacts/credentials-for-read\x1a\x04\x08\x02\x10\x00\x10\x03\x12\xa0\x01\n\x16getCredentialsForWrite\x12\x1e.mlflow.GetCredentialsForWrite\x1a\'.mlflow.GetCredentialsForWrite.Response\"=\xf2\x86\x19\x39\n5\n\x04POST\x12\'/mlflow/artifacts/credentials-for-write\x1a\x04\x08\x02\x10\x00\x10\x03\x12\x9f\x01\n\x15\x63reateMultipartUpload\x12\x1d.mlflow.CreateMultipartUpload\x1a&.mlflow.CreateMultipartUpload.Response\"?\xf2\x86\x19;\n7\n\x04POST\x12)/mlflow/artifacts/create-multipart-upload\x1a\x04\x08\x02\x10\x00\x10\x03\x12\xa7\x01\n\x17\x63ompleteMultipartUpload\x12\x1f.mlflow.CompleteMultipartUpload\x1a(.mlflow.CompleteMultipartUpload.Response\"A\xf2\x86\x19=\n9\n\x04POST\x12+/mlflow/artifacts/complete-multipart-upload\x1a\x04\x08\x02\x10\x00\x10\x03\x12\xb0\x01\n\x19getPresignedUploadPartUrl\x12!.mlflow.GetPresignedUploadPartUrl\x1a*.mlflow.GetPresignedUploadPartUrl.Response\"D\xf2\x86\x19@\n<\n\x03GET\x12//mlflow/artifacts/get-presigned-upload-part-url\x1a\x04\x08\x02\x10\x00\x10\x03\x12\xd5\x01\n\"getCredentialsForTraceDataDownload\x12*.mlflow.GetCredentialsForTraceDataDownload\x1a\x33.mlflow.GetCredentialsForTraceDataDownload.Response\"N\xf2\x86\x19J\nF\n\x03GET\x12\x39/mlflow/traces/{request_id}/credentials-for-data-download\x1a\x04\x08\x02\x10\x00\x10\x03\x12\xcd\x01\n getCredentialsForTraceDataUpload\x12(.mlflow.GetCredentialsForTraceDataUpload\x1a\x31.mlflow.GetCredentialsForTraceDataUpload.Response\"L\xf2\x86\x19H\nD\n\x03GET\x12\x37/mlflow/traces/{request_id}/credentials-for-data-upload\x1a\x04\x08\x02\x10\x00\x10\x03\x12\xde\x01\n\"getCredentialsForLoggedModelUpload\x12*.mlflow.GetCredentialsForLoggedModelUpload\x1a\x33.mlflow.GetCredentialsForLoggedModelUpload.Response\"W\xf2\x86\x19S\nO\n\x04POST\x12\x41/mlflow/logged-models/{model_id}/artifacts/credentials-for-upload\x1a\x04\x08\x02\x10\x00\x10\x03\x12\xe6\x01\n$getCredentialsForLoggedModelDownload\x12,.mlflow.GetCredentialsForLoggedModelDownload\x1a\x35.mlflow.GetCredentialsForLoggedModelDownload.Response\"Y\xf2\x86\x19U\nQ\n\x04POST\x12\x43/mlflow/logged-models/{model_id}/artifacts/credentials-for-download\x1a\x04\x08\x02\x10\x00\x10\x03\x42,\n\x1f\x63om.databricks.api.proto.mlflow\x90\x01\x01\xa0\x01\x01\xe2?\x02\x10\x01')

  _ARTIFACTCREDENTIALTYPE = DESCRIPTOR.enum_types_by_name['ArtifactCredentialType']
  ArtifactCredentialType = enum_type_wrapper.EnumTypeWrapper(_ARTIFACTCREDENTIALTYPE)
  AZURE_SAS_URI = 1
  AWS_PRESIGNED_URL = 2
  GCP_SIGNED_URL = 3
  AZURE_ADLS_GEN2_SAS_URI = 4


  _ARTIFACTCREDENTIALINFO = DESCRIPTOR.message_types_by_name['ArtifactCredentialInfo']
  _ARTIFACTCREDENTIALINFO_HTTPHEADER = _ARTIFACTCREDENTIALINFO.nested_types_by_name['HttpHeader']
  _LOGGEDMODELARTIFACTCREDENTIAL = DESCRIPTOR.message_types_by_name['LoggedModelArtifactCredential']
  _GETCREDENTIALSFORREAD = DESCRIPTOR.message_types_by_name['GetCredentialsForRead']
  _GETCREDENTIALSFORREAD_RESPONSE = _GETCREDENTIALSFORREAD.nested_types_by_name['Response']
  _GETCREDENTIALSFORWRITE = DESCRIPTOR.message_types_by_name['GetCredentialsForWrite']
  _GETCREDENTIALSFORWRITE_RESPONSE = _GETCREDENTIALSFORWRITE.nested_types_by_name['Response']
  _CREATEMULTIPARTUPLOAD = DESCRIPTOR.message_types_by_name['CreateMultipartUpload']
  _CREATEMULTIPARTUPLOAD_RESPONSE = _CREATEMULTIPARTUPLOAD.nested_types_by_name['Response']
  _PARTETAG = DESCRIPTOR.message_types_by_name['PartEtag']
  _COMPLETEMULTIPARTUPLOAD = DESCRIPTOR.message_types_by_name['CompleteMultipartUpload']
  _COMPLETEMULTIPARTUPLOAD_RESPONSE = _COMPLETEMULTIPARTUPLOAD.nested_types_by_name['Response']
  _GETPRESIGNEDUPLOADPARTURL = DESCRIPTOR.message_types_by_name['GetPresignedUploadPartUrl']
  _GETPRESIGNEDUPLOADPARTURL_RESPONSE = _GETPRESIGNEDUPLOADPARTURL.nested_types_by_name['Response']
  _GETCREDENTIALSFORTRACEDATADOWNLOAD = DESCRIPTOR.message_types_by_name['GetCredentialsForTraceDataDownload']
  _GETCREDENTIALSFORTRACEDATADOWNLOAD_RESPONSE = _GETCREDENTIALSFORTRACEDATADOWNLOAD.nested_types_by_name['Response']
  _GETCREDENTIALSFORTRACEDATAUPLOAD = DESCRIPTOR.message_types_by_name['GetCredentialsForTraceDataUpload']
  _GETCREDENTIALSFORTRACEDATAUPLOAD_RESPONSE = _GETCREDENTIALSFORTRACEDATAUPLOAD.nested_types_by_name['Response']
  _GETCREDENTIALSFORLOGGEDMODELUPLOAD = DESCRIPTOR.message_types_by_name['GetCredentialsForLoggedModelUpload']
  _GETCREDENTIALSFORLOGGEDMODELUPLOAD_RESPONSE = _GETCREDENTIALSFORLOGGEDMODELUPLOAD.nested_types_by_name['Response']
  _GETCREDENTIALSFORLOGGEDMODELDOWNLOAD = DESCRIPTOR.message_types_by_name['GetCredentialsForLoggedModelDownload']
  _GETCREDENTIALSFORLOGGEDMODELDOWNLOAD_RESPONSE = _GETCREDENTIALSFORLOGGEDMODELDOWNLOAD.nested_types_by_name['Response']
  ArtifactCredentialInfo = _reflection.GeneratedProtocolMessageType('ArtifactCredentialInfo', (_message.Message,), {

    'HttpHeader' : _reflection.GeneratedProtocolMessageType('HttpHeader', (_message.Message,), {
      'DESCRIPTOR' : _ARTIFACTCREDENTIALINFO_HTTPHEADER,
      '__module__' : 'databricks_artifacts_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.ArtifactCredentialInfo.HttpHeader)
      })
    ,
    'DESCRIPTOR' : _ARTIFACTCREDENTIALINFO,
    '__module__' : 'databricks_artifacts_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.ArtifactCredentialInfo)
    })
  _sym_db.RegisterMessage(ArtifactCredentialInfo)
  _sym_db.RegisterMessage(ArtifactCredentialInfo.HttpHeader)

  LoggedModelArtifactCredential = _reflection.GeneratedProtocolMessageType('LoggedModelArtifactCredential', (_message.Message,), {
    'DESCRIPTOR' : _LOGGEDMODELARTIFACTCREDENTIAL,
    '__module__' : 'databricks_artifacts_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.LoggedModelArtifactCredential)
    })
  _sym_db.RegisterMessage(LoggedModelArtifactCredential)

  GetCredentialsForRead = _reflection.GeneratedProtocolMessageType('GetCredentialsForRead', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _GETCREDENTIALSFORREAD_RESPONSE,
      '__module__' : 'databricks_artifacts_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.GetCredentialsForRead.Response)
      })
    ,
    'DESCRIPTOR' : _GETCREDENTIALSFORREAD,
    '__module__' : 'databricks_artifacts_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.GetCredentialsForRead)
    })
  _sym_db.RegisterMessage(GetCredentialsForRead)
  _sym_db.RegisterMessage(GetCredentialsForRead.Response)

  GetCredentialsForWrite = _reflection.GeneratedProtocolMessageType('GetCredentialsForWrite', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _GETCREDENTIALSFORWRITE_RESPONSE,
      '__module__' : 'databricks_artifacts_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.GetCredentialsForWrite.Response)
      })
    ,
    'DESCRIPTOR' : _GETCREDENTIALSFORWRITE,
    '__module__' : 'databricks_artifacts_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.GetCredentialsForWrite)
    })
  _sym_db.RegisterMessage(GetCredentialsForWrite)
  _sym_db.RegisterMessage(GetCredentialsForWrite.Response)

  CreateMultipartUpload = _reflection.GeneratedProtocolMessageType('CreateMultipartUpload', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _CREATEMULTIPARTUPLOAD_RESPONSE,
      '__module__' : 'databricks_artifacts_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.CreateMultipartUpload.Response)
      })
    ,
    'DESCRIPTOR' : _CREATEMULTIPARTUPLOAD,
    '__module__' : 'databricks_artifacts_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.CreateMultipartUpload)
    })
  _sym_db.RegisterMessage(CreateMultipartUpload)
  _sym_db.RegisterMessage(CreateMultipartUpload.Response)

  PartEtag = _reflection.GeneratedProtocolMessageType('PartEtag', (_message.Message,), {
    'DESCRIPTOR' : _PARTETAG,
    '__module__' : 'databricks_artifacts_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.PartEtag)
    })
  _sym_db.RegisterMessage(PartEtag)

  CompleteMultipartUpload = _reflection.GeneratedProtocolMessageType('CompleteMultipartUpload', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _COMPLETEMULTIPARTUPLOAD_RESPONSE,
      '__module__' : 'databricks_artifacts_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.CompleteMultipartUpload.Response)
      })
    ,
    'DESCRIPTOR' : _COMPLETEMULTIPARTUPLOAD,
    '__module__' : 'databricks_artifacts_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.CompleteMultipartUpload)
    })
  _sym_db.RegisterMessage(CompleteMultipartUpload)
  _sym_db.RegisterMessage(CompleteMultipartUpload.Response)

  GetPresignedUploadPartUrl = _reflection.GeneratedProtocolMessageType('GetPresignedUploadPartUrl', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _GETPRESIGNEDUPLOADPARTURL_RESPONSE,
      '__module__' : 'databricks_artifacts_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.GetPresignedUploadPartUrl.Response)
      })
    ,
    'DESCRIPTOR' : _GETPRESIGNEDUPLOADPARTURL,
    '__module__' : 'databricks_artifacts_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.GetPresignedUploadPartUrl)
    })
  _sym_db.RegisterMessage(GetPresignedUploadPartUrl)
  _sym_db.RegisterMessage(GetPresignedUploadPartUrl.Response)

  GetCredentialsForTraceDataDownload = _reflection.GeneratedProtocolMessageType('GetCredentialsForTraceDataDownload', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _GETCREDENTIALSFORTRACEDATADOWNLOAD_RESPONSE,
      '__module__' : 'databricks_artifacts_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.GetCredentialsForTraceDataDownload.Response)
      })
    ,
    'DESCRIPTOR' : _GETCREDENTIALSFORTRACEDATADOWNLOAD,
    '__module__' : 'databricks_artifacts_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.GetCredentialsForTraceDataDownload)
    })
  _sym_db.RegisterMessage(GetCredentialsForTraceDataDownload)
  _sym_db.RegisterMessage(GetCredentialsForTraceDataDownload.Response)

  GetCredentialsForTraceDataUpload = _reflection.GeneratedProtocolMessageType('GetCredentialsForTraceDataUpload', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _GETCREDENTIALSFORTRACEDATAUPLOAD_RESPONSE,
      '__module__' : 'databricks_artifacts_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.GetCredentialsForTraceDataUpload.Response)
      })
    ,
    'DESCRIPTOR' : _GETCREDENTIALSFORTRACEDATAUPLOAD,
    '__module__' : 'databricks_artifacts_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.GetCredentialsForTraceDataUpload)
    })
  _sym_db.RegisterMessage(GetCredentialsForTraceDataUpload)
  _sym_db.RegisterMessage(GetCredentialsForTraceDataUpload.Response)

  GetCredentialsForLoggedModelUpload = _reflection.GeneratedProtocolMessageType('GetCredentialsForLoggedModelUpload', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _GETCREDENTIALSFORLOGGEDMODELUPLOAD_RESPONSE,
      '__module__' : 'databricks_artifacts_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.GetCredentialsForLoggedModelUpload.Response)
      })
    ,
    'DESCRIPTOR' : _GETCREDENTIALSFORLOGGEDMODELUPLOAD,
    '__module__' : 'databricks_artifacts_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.GetCredentialsForLoggedModelUpload)
    })
  _sym_db.RegisterMessage(GetCredentialsForLoggedModelUpload)
  _sym_db.RegisterMessage(GetCredentialsForLoggedModelUpload.Response)

  GetCredentialsForLoggedModelDownload = _reflection.GeneratedProtocolMessageType('GetCredentialsForLoggedModelDownload', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _GETCREDENTIALSFORLOGGEDMODELDOWNLOAD_RESPONSE,
      '__module__' : 'databricks_artifacts_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.GetCredentialsForLoggedModelDownload.Response)
      })
    ,
    'DESCRIPTOR' : _GETCREDENTIALSFORLOGGEDMODELDOWNLOAD,
    '__module__' : 'databricks_artifacts_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.GetCredentialsForLoggedModelDownload)
    })
  _sym_db.RegisterMessage(GetCredentialsForLoggedModelDownload)
  _sym_db.RegisterMessage(GetCredentialsForLoggedModelDownload.Response)

  _DATABRICKSMLFLOWARTIFACTSSERVICE = DESCRIPTOR.services_by_name['DatabricksMlflowArtifactsService']
  if _descriptor._USE_C_DESCRIPTORS == False:

    DESCRIPTOR._options = None
    DESCRIPTOR._serialized_options = b'\n\037com.databricks.api.proto.mlflow\220\001\001\240\001\001\342?\002\020\001'
    _GETCREDENTIALSFORREAD.fields_by_name['run_id']._options = None
    _GETCREDENTIALSFORREAD.fields_by_name['run_id']._serialized_options = b'\370\206\031\001'
    _GETCREDENTIALSFORREAD._options = None
    _GETCREDENTIALSFORREAD._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _GETCREDENTIALSFORWRITE.fields_by_name['run_id']._options = None
    _GETCREDENTIALSFORWRITE.fields_by_name['run_id']._serialized_options = b'\370\206\031\001'
    _GETCREDENTIALSFORWRITE._options = None
    _GETCREDENTIALSFORWRITE._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _CREATEMULTIPARTUPLOAD.fields_by_name['run_id']._options = None
    _CREATEMULTIPARTUPLOAD.fields_by_name['run_id']._serialized_options = b'\370\206\031\001'
    _CREATEMULTIPARTUPLOAD.fields_by_name['num_parts']._options = None
    _CREATEMULTIPARTUPLOAD.fields_by_name['num_parts']._serialized_options = b'\370\206\031\001'
    _CREATEMULTIPARTUPLOAD._options = None
    _CREATEMULTIPARTUPLOAD._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _COMPLETEMULTIPARTUPLOAD.fields_by_name['run_id']._options = None
    _COMPLETEMULTIPARTUPLOAD.fields_by_name['run_id']._serialized_options = b'\370\206\031\001'
    _COMPLETEMULTIPARTUPLOAD.fields_by_name['upload_id']._options = None
    _COMPLETEMULTIPARTUPLOAD.fields_by_name['upload_id']._serialized_options = b'\370\206\031\001'
    _COMPLETEMULTIPARTUPLOAD._options = None
    _COMPLETEMULTIPARTUPLOAD._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _GETPRESIGNEDUPLOADPARTURL.fields_by_name['run_id']._options = None
    _GETPRESIGNEDUPLOADPARTURL.fields_by_name['run_id']._serialized_options = b'\370\206\031\001'
    _GETPRESIGNEDUPLOADPARTURL.fields_by_name['upload_id']._options = None
    _GETPRESIGNEDUPLOADPARTURL.fields_by_name['upload_id']._serialized_options = b'\370\206\031\001'
    _GETPRESIGNEDUPLOADPARTURL.fields_by_name['part_number']._options = None
    _GETPRESIGNEDUPLOADPARTURL.fields_by_name['part_number']._serialized_options = b'\370\206\031\001'
    _GETPRESIGNEDUPLOADPARTURL._options = None
    _GETPRESIGNEDUPLOADPARTURL._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _GETCREDENTIALSFORTRACEDATADOWNLOAD.fields_by_name['request_id']._options = None
    _GETCREDENTIALSFORTRACEDATADOWNLOAD.fields_by_name['request_id']._serialized_options = b'\370\206\031\001'
    _GETCREDENTIALSFORTRACEDATADOWNLOAD._options = None
    _GETCREDENTIALSFORTRACEDATADOWNLOAD._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _GETCREDENTIALSFORTRACEDATAUPLOAD.fields_by_name['request_id']._options = None
    _GETCREDENTIALSFORTRACEDATAUPLOAD.fields_by_name['request_id']._serialized_options = b'\370\206\031\001'
    _GETCREDENTIALSFORTRACEDATAUPLOAD._options = None
    _GETCREDENTIALSFORTRACEDATAUPLOAD._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _GETCREDENTIALSFORLOGGEDMODELUPLOAD.fields_by_name['model_id']._options = None
    _GETCREDENTIALSFORLOGGEDMODELUPLOAD.fields_by_name['model_id']._serialized_options = b'\370\206\031\001'
    _GETCREDENTIALSFORLOGGEDMODELUPLOAD._options = None
    _GETCREDENTIALSFORLOGGEDMODELUPLOAD._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _GETCREDENTIALSFORLOGGEDMODELDOWNLOAD.fields_by_name['model_id']._options = None
    _GETCREDENTIALSFORLOGGEDMODELDOWNLOAD.fields_by_name['model_id']._serialized_options = b'\370\206\031\001'
    _GETCREDENTIALSFORLOGGEDMODELDOWNLOAD._options = None
    _GETCREDENTIALSFORLOGGEDMODELDOWNLOAD._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _DATABRICKSMLFLOWARTIFACTSSERVICE.methods_by_name['getCredentialsForRead']._options = None
    _DATABRICKSMLFLOWARTIFACTSSERVICE.methods_by_name['getCredentialsForRead']._serialized_options = b'\362\206\0318\n4\n\004POST\022&/mlflow/artifacts/credentials-for-read\032\004\010\002\020\000\020\003'
    _DATABRICKSMLFLOWARTIFACTSSERVICE.methods_by_name['getCredentialsForWrite']._options = None
    _DATABRICKSMLFLOWARTIFACTSSERVICE.methods_by_name['getCredentialsForWrite']._serialized_options = b'\362\206\0319\n5\n\004POST\022\'/mlflow/artifacts/credentials-for-write\032\004\010\002\020\000\020\003'
    _DATABRICKSMLFLOWARTIFACTSSERVICE.methods_by_name['createMultipartUpload']._options = None
    _DATABRICKSMLFLOWARTIFACTSSERVICE.methods_by_name['createMultipartUpload']._serialized_options = b'\362\206\031;\n7\n\004POST\022)/mlflow/artifacts/create-multipart-upload\032\004\010\002\020\000\020\003'
    _DATABRICKSMLFLOWARTIFACTSSERVICE.methods_by_name['completeMultipartUpload']._options = None
    _DATABRICKSMLFLOWARTIFACTSSERVICE.methods_by_name['completeMultipartUpload']._serialized_options = b'\362\206\031=\n9\n\004POST\022+/mlflow/artifacts/complete-multipart-upload\032\004\010\002\020\000\020\003'
    _DATABRICKSMLFLOWARTIFACTSSERVICE.methods_by_name['getPresignedUploadPartUrl']._options = None
    _DATABRICKSMLFLOWARTIFACTSSERVICE.methods_by_name['getPresignedUploadPartUrl']._serialized_options = b'\362\206\031@\n<\n\003GET\022//mlflow/artifacts/get-presigned-upload-part-url\032\004\010\002\020\000\020\003'
    _DATABRICKSMLFLOWARTIFACTSSERVICE.methods_by_name['getCredentialsForTraceDataDownload']._options = None
    _DATABRICKSMLFLOWARTIFACTSSERVICE.methods_by_name['getCredentialsForTraceDataDownload']._serialized_options = b'\362\206\031J\nF\n\003GET\0229/mlflow/traces/{request_id}/credentials-for-data-download\032\004\010\002\020\000\020\003'
    _DATABRICKSMLFLOWARTIFACTSSERVICE.methods_by_name['getCredentialsForTraceDataUpload']._options = None
    _DATABRICKSMLFLOWARTIFACTSSERVICE.methods_by_name['getCredentialsForTraceDataUpload']._serialized_options = b'\362\206\031H\nD\n\003GET\0227/mlflow/traces/{request_id}/credentials-for-data-upload\032\004\010\002\020\000\020\003'
    _DATABRICKSMLFLOWARTIFACTSSERVICE.methods_by_name['getCredentialsForLoggedModelUpload']._options = None
    _DATABRICKSMLFLOWARTIFACTSSERVICE.methods_by_name['getCredentialsForLoggedModelUpload']._serialized_options = b'\362\206\031S\nO\n\004POST\022A/mlflow/logged-models/{model_id}/artifacts/credentials-for-upload\032\004\010\002\020\000\020\003'
    _DATABRICKSMLFLOWARTIFACTSSERVICE.methods_by_name['getCredentialsForLoggedModelDownload']._options = None
    _DATABRICKSMLFLOWARTIFACTSSERVICE.methods_by_name['getCredentialsForLoggedModelDownload']._serialized_options = b'\362\206\031U\nQ\n\004POST\022C/mlflow/logged-models/{model_id}/artifacts/credentials-for-download\032\004\010\002\020\000\020\003'
    _ARTIFACTCREDENTIALTYPE._serialized_start=2470
    _ARTIFACTCREDENTIALTYPE._serialized_end=2585
    _ARTIFACTCREDENTIALINFO._serialized_start=80
    _ARTIFACTCREDENTIALINFO._serialized_end=303
    _ARTIFACTCREDENTIALINFO_HTTPHEADER._serialized_start=262
    _ARTIFACTCREDENTIALINFO_HTTPHEADER._serialized_end=303
    _LOGGEDMODELARTIFACTCREDENTIAL._serialized_start=305
    _LOGGEDMODELARTIFACTCREDENTIAL._serialized_end=411
    _GETCREDENTIALSFORREAD._serialized_start=414
    _GETCREDENTIALSFORREAD._serialized_end=639
    _GETCREDENTIALSFORREAD_RESPONSE._serialized_start=495
    _GETCREDENTIALSFORREAD_RESPONSE._serialized_end=594
    _GETCREDENTIALSFORWRITE._serialized_start=642
    _GETCREDENTIALSFORWRITE._serialized_end=868
    _GETCREDENTIALSFORWRITE_RESPONSE._serialized_start=495
    _GETCREDENTIALSFORWRITE_RESPONSE._serialized_end=594
    _CREATEMULTIPARTUPLOAD._serialized_start=871
    _CREATEMULTIPARTUPLOAD._serialized_end=1160
    _CREATEMULTIPARTUPLOAD_RESPONSE._serialized_start=958
    _CREATEMULTIPARTUPLOAD_RESPONSE._serialized_end=1115
    _PARTETAG._serialized_start=1162
    _PARTETAG._serialized_end=1207
    _COMPLETEMULTIPARTUPLOAD._serialized_start=1210
    _COMPLETEMULTIPARTUPLOAD._serialized_end=1391
    _COMPLETEMULTIPARTUPLOAD_RESPONSE._serialized_start=495
    _COMPLETEMULTIPARTUPLOAD_RESPONSE._serialized_end=505
    _GETPRESIGNEDUPLOADPARTURL._serialized_start=1394
    _GETPRESIGNEDUPLOADPARTURL._serialized_end=1630
    _GETPRESIGNEDUPLOADPARTURL_RESPONSE._serialized_start=1511
    _GETPRESIGNEDUPLOADPARTURL_RESPONSE._serialized_end=1585
    _GETCREDENTIALSFORTRACEDATADOWNLOAD._serialized_start=1633
    _GETCREDENTIALSFORTRACEDATADOWNLOAD._serialized_end=1809
    _GETCREDENTIALSFORTRACEDATADOWNLOAD_RESPONSE._serialized_start=1697
    _GETCREDENTIALSFORTRACEDATADOWNLOAD_RESPONSE._serialized_end=1764
    _GETCREDENTIALSFORTRACEDATAUPLOAD._serialized_start=1812
    _GETCREDENTIALSFORTRACEDATAUPLOAD._serialized_end=1986
    _GETCREDENTIALSFORTRACEDATAUPLOAD_RESPONSE._serialized_start=1697
    _GETCREDENTIALSFORTRACEDATAUPLOAD_RESPONSE._serialized_end=1764
    _GETCREDENTIALSFORLOGGEDMODELUPLOAD._serialized_start=1989
    _GETCREDENTIALSFORLOGGEDMODELUPLOAD._serialized_end=2226
    _GETCREDENTIALSFORLOGGEDMODELUPLOAD_RESPONSE._serialized_start=2086
    _GETCREDENTIALSFORLOGGEDMODELUPLOAD_RESPONSE._serialized_end=2181
    _GETCREDENTIALSFORLOGGEDMODELDOWNLOAD._serialized_start=2229
    _GETCREDENTIALSFORLOGGEDMODELDOWNLOAD._serialized_end=2468
    _GETCREDENTIALSFORLOGGEDMODELDOWNLOAD_RESPONSE._serialized_start=2086
    _GETCREDENTIALSFORLOGGEDMODELDOWNLOAD_RESPONSE._serialized_end=2181
    _DATABRICKSMLFLOWARTIFACTSSERVICE._serialized_start=2588
    _DATABRICKSMLFLOWARTIFACTSSERVICE._serialized_end=4337
  DatabricksMlflowArtifactsService = service_reflection.GeneratedServiceType('DatabricksMlflowArtifactsService', (_service.Service,), dict(
    DESCRIPTOR = _DATABRICKSMLFLOWARTIFACTSSERVICE,
    __module__ = 'databricks_artifacts_pb2'
    ))

  DatabricksMlflowArtifactsService_Stub = service_reflection.GeneratedServiceStubType('DatabricksMlflowArtifactsService_Stub', (DatabricksMlflowArtifactsService,), dict(
    DESCRIPTOR = _DATABRICKSMLFLOWARTIFACTSSERVICE,
    __module__ = 'databricks_artifacts_pb2'
    ))


  # @@protoc_insertion_point(module_scope)




from mlflow.exceptions import MlflowException
from mlflow.protos import databricks_pb2
from mlflow.tracking.client import MlflowClient
from mlflow.tracking.default_experiment.abstract_context import DefaultExperimentProvider
from mlflow.utils import databricks_utils
from mlflow.utils.mlflow_tags import MLFLOW_EXPERIMENT_SOURCE_ID, MLFLOW_EXPERIMENT_SOURCE_TYPE


class DatabricksNotebookExperimentProvider(DefaultExperimentProvider):
    _resolved_notebook_experiment_id = None

    def in_context(self):
        return databricks_utils.is_in_databricks_notebook()

    def get_experiment_id(self):
        if DatabricksNotebookExperimentProvider._resolved_notebook_experiment_id:
            return DatabricksNotebookExperimentProvider._resolved_notebook_experiment_id

        source_notebook_id = databricks_utils.get_notebook_id()
        source_notebook_name = databricks_utils.get_notebook_path()
        tags = {
            MLFLOW_EXPERIMENT_SOURCE_ID: source_notebook_id,
        }

        if databricks_utils.is_in_databricks_repo_notebook():
            tags[MLFLOW_EXPERIMENT_SOURCE_TYPE] = "REPO_NOTEBOOK"

        # With the presence of the source id, the following is a get or create in which it will
        # return the corresponding experiment if one exists for the repo notebook.
        # For non-repo notebooks, it will raise an exception and we will use source_notebook_id
        try:
            experiment_id = MlflowClient().create_experiment(source_notebook_name, None, tags)
        except MlflowException as e:
            if e.error_code == databricks_pb2.ErrorCode.Name(
                databricks_pb2.INVALID_PARAMETER_VALUE
            ):
                # If determined that it is not a repo notebook
                experiment_id = source_notebook_id
            else:
                raise e

        DatabricksNotebookExperimentProvider._resolved_notebook_experiment_id = experiment_id

        return experiment_id




import google.protobuf
from packaging.version import Version
if Version(google.protobuf.__version__).major >= 5:
  # -*- coding: utf-8 -*-
  # Generated by the protocol buffer compiler.  DO NOT EDIT!
  # source: databricks_managed_catalog_service.proto
  # Protobuf Python Version: 5.26.0
  """Generated protocol buffer code."""
  from google.protobuf import descriptor as _descriptor
  from google.protobuf import descriptor_pool as _descriptor_pool
  from google.protobuf import symbol_database as _symbol_database
  from google.protobuf.internal import builder as _builder
  # @@protoc_insertion_point(imports)

  _sym_db = _symbol_database.Default()


  from . import databricks_pb2 as databricks__pb2
  from . import databricks_managed_catalog_messages_pb2 as databricks_managed_catalog_messages_pb2
  from .scalapb import scalapb_pb2 as scalapb_dot_scalapb__pb2


  DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n(databricks_managed_catalog_service.proto\x12\x15mlflow.managedcatalog\x1a\x10\x64\x61tabricks.proto\x1a)databricks_managed_catalog_messages.proto\x1a\x15scalapb/scalapb.proto2\xb2\x01\n\x1d\x44\x61tabricksUnityCatalogService\x12\x90\x01\n\x08getTable\x12\x1f.mlflow.managedcatalog.GetTable\x1a\'.mlflow.managedcatalog.GetTableResponse\":\xf2\x86\x19\x36\n2\n\x03GET\x12%/unity-catalog/tables/{full_name_arg}\x1a\x04\x08\x02\x10\x00\x10\x03\x42\x34\n\'com.databricks.api.proto.managedcatalog\x90\x01\x01\xa0\x01\x01\xe2?\x02\x10\x01')

  _globals = globals()
  _builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
  _builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'databricks_managed_catalog_service_pb2', _globals)
  if not _descriptor._USE_C_DESCRIPTORS:
    _globals['DESCRIPTOR']._loaded_options = None
    _globals['DESCRIPTOR']._serialized_options = b'\n\'com.databricks.api.proto.managedcatalog\220\001\001\240\001\001\342?\002\020\001'
    _globals['_DATABRICKSUNITYCATALOGSERVICE'].methods_by_name['getTable']._loaded_options = None
    _globals['_DATABRICKSUNITYCATALOGSERVICE'].methods_by_name['getTable']._serialized_options = b'\362\206\0316\n2\n\003GET\022%/unity-catalog/tables/{full_name_arg}\032\004\010\002\020\000\020\003'
    _globals['_DATABRICKSUNITYCATALOGSERVICE']._serialized_start=152
    _globals['_DATABRICKSUNITYCATALOGSERVICE']._serialized_end=330
  _builder.BuildServices(DESCRIPTOR, 'databricks_managed_catalog_service_pb2', _globals)
  # @@protoc_insertion_point(module_scope)

else:
  # -*- coding: utf-8 -*-
  # Generated by the protocol buffer compiler.  DO NOT EDIT!
  # source: databricks_managed_catalog_service.proto
  """Generated protocol buffer code."""
  from google.protobuf import descriptor as _descriptor
  from google.protobuf import descriptor_pool as _descriptor_pool
  from google.protobuf import message as _message
  from google.protobuf import reflection as _reflection
  from google.protobuf import symbol_database as _symbol_database
  from google.protobuf import service as _service
  from google.protobuf import service_reflection
  # @@protoc_insertion_point(imports)

  _sym_db = _symbol_database.Default()


  from . import databricks_pb2 as databricks__pb2
  from . import databricks_managed_catalog_messages_pb2 as databricks_managed_catalog_messages_pb2
  from .scalapb import scalapb_pb2 as scalapb_dot_scalapb__pb2


  DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n(databricks_managed_catalog_service.proto\x12\x15mlflow.managedcatalog\x1a\x10\x64\x61tabricks.proto\x1a)databricks_managed_catalog_messages.proto\x1a\x15scalapb/scalapb.proto2\xb2\x01\n\x1d\x44\x61tabricksUnityCatalogService\x12\x90\x01\n\x08getTable\x12\x1f.mlflow.managedcatalog.GetTable\x1a\'.mlflow.managedcatalog.GetTableResponse\":\xf2\x86\x19\x36\n2\n\x03GET\x12%/unity-catalog/tables/{full_name_arg}\x1a\x04\x08\x02\x10\x00\x10\x03\x42\x34\n\'com.databricks.api.proto.managedcatalog\x90\x01\x01\xa0\x01\x01\xe2?\x02\x10\x01')



  _DATABRICKSUNITYCATALOGSERVICE = DESCRIPTOR.services_by_name['DatabricksUnityCatalogService']
  if _descriptor._USE_C_DESCRIPTORS == False:

    DESCRIPTOR._options = None
    DESCRIPTOR._serialized_options = b'\n\'com.databricks.api.proto.managedcatalog\220\001\001\240\001\001\342?\002\020\001'
    _DATABRICKSUNITYCATALOGSERVICE.methods_by_name['getTable']._options = None
    _DATABRICKSUNITYCATALOGSERVICE.methods_by_name['getTable']._serialized_options = b'\362\206\0316\n2\n\003GET\022%/unity-catalog/tables/{full_name_arg}\032\004\010\002\020\000\020\003'
    _DATABRICKSUNITYCATALOGSERVICE._serialized_start=152
    _DATABRICKSUNITYCATALOGSERVICE._serialized_end=330
  DatabricksUnityCatalogService = service_reflection.GeneratedServiceType('DatabricksUnityCatalogService', (_service.Service,), dict(
    DESCRIPTOR = _DATABRICKSUNITYCATALOGSERVICE,
    __module__ = 'databricks_managed_catalog_service_pb2'
    ))

  DatabricksUnityCatalogService_Stub = service_reflection.GeneratedServiceStubType('DatabricksUnityCatalogService_Stub', (DatabricksUnityCatalogService,), dict(
    DESCRIPTOR = _DATABRICKSUNITYCATALOGSERVICE,
    __module__ = 'databricks_managed_catalog_service_pb2'
    ))


  # @@protoc_insertion_point(module_scope)




import io
import json
import logging
from typing import Any, Optional, Union

from botocore.client import BaseClient
from botocore.response import StreamingBody

import mlflow
from mlflow.bedrock import FLAVOR_NAME
from mlflow.bedrock.chat import convert_message_to_mlflow_chat, convert_tool_to_mlflow_chat_tool
from mlflow.bedrock.stream import ConverseStreamWrapper, InvokeModelStreamWrapper
from mlflow.bedrock.utils import skip_if_trace_disabled
from mlflow.entities import SpanType
from mlflow.tracing.fluent import start_span_no_context
from mlflow.tracing.utils import set_span_chat_messages, set_span_chat_tools
from mlflow.utils.autologging_utils import safe_patch

_BEDROCK_RUNTIME_SERVICE_NAME = "bedrock-runtime"
_BEDROCK_SPAN_PREFIX = "BedrockRuntime."

_logger = logging.getLogger(__name__)


def patched_create_client(original, self, *args, **kwargs):
    """
    Patched version of the boto3 ClientCreator.create_client method that returns
    a patched client class.
    """
    if kwargs.get("service_name") != _BEDROCK_RUNTIME_SERVICE_NAME:
        return original(self, *args, **kwargs)

    client = original(self, *args, **kwargs)
    patch_bedrock_runtime_client(client.__class__)

    return client


def patch_bedrock_runtime_client(client_class: type[BaseClient]):
    """
    Patch the BedrockRuntime client to log traces and models.
    """
    # The most basic model invocation API
    safe_patch(FLAVOR_NAME, client_class, "invoke_model", _patched_invoke_model)
    safe_patch(
        FLAVOR_NAME,
        client_class,
        "invoke_model_with_response_stream",
        _patched_invoke_model_with_response_stream,
    )

    if hasattr(client_class, "converse"):
        # The new "converse" API was introduced in boto3 1.35 to access all models
        # with the consistent chat format.
        # https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/bedrock-runtime/client/converse.html
        safe_patch(FLAVOR_NAME, client_class, "converse", _patched_converse)

    if hasattr(client_class, "converse_stream"):
        safe_patch(FLAVOR_NAME, client_class, "converse_stream", _patched_converse_stream)


@skip_if_trace_disabled
def _patched_invoke_model(original, self, *args, **kwargs):
    with mlflow.start_span(name=f"{_BEDROCK_SPAN_PREFIX}{original.__name__}") as span:
        # NB: Bedrock client doesn't accept any positional arguments
        span.set_inputs(kwargs)

        result = original(self, *args, **kwargs)

        result["body"] = _buffer_stream(result["body"])
        parsed_response_body = _parse_invoke_model_response_body(result["body"])

        # Determine the span type based on the key in the response body.
        # As of 2024 Dec 9th, all supported embedding models in Bedrock returns the response body
        # with the key "embedding". This might change in the future.
        span_type = SpanType.EMBEDDING if "embedding" in parsed_response_body else SpanType.LLM
        span.set_span_type(span_type)
        span.set_outputs({**result, "body": parsed_response_body})

        return result


@skip_if_trace_disabled
def _patched_invoke_model_with_response_stream(original, self, *args, **kwargs):
    span = start_span_no_context(
        name=f"{_BEDROCK_SPAN_PREFIX}{original.__name__}",
        # NB: Since we don't inspect the response body for this method, the span type is unknown.
        # We assume it is LLM as using streaming for embedding is not common.
        span_type=SpanType.LLM,
        inputs=kwargs,
    )

    result = original(self, *args, **kwargs)

    # To avoid consuming the stream during serialization, set dummy outputs for the span.
    span.set_outputs({**result, "body": "EventStream"})

    result["body"] = InvokeModelStreamWrapper(stream=result["body"], span=span)
    return result


def _buffer_stream(raw_stream: StreamingBody) -> StreamingBody:
    """
    Create a buffered stream from the raw byte stream.

    The boto3's invoke_model() API returns the LLM response as a byte stream.
    We need to read the stream data to set the span outputs, however, the stream
    can only be read once and not seekable (https://github.com/boto/boto3/issues/564).
    To work around this, we create a buffered stream that can be read multiple times.
    """
    buffered_response = io.BytesIO(raw_stream.read())
    buffered_response.seek(0)
    return StreamingBody(buffered_response, raw_stream._content_length)


def _parse_invoke_model_response_body(response_body: StreamingBody) -> Union[dict[str, Any], str]:
    content = response_body.read()
    try:
        return json.loads(content)
    except Exception:
        # When failed to parse the response body as JSON, return the raw response
        return content
    finally:
        # Reset the stream position to the beginning
        response_body._raw_stream.seek(0)
        # Boto3 uses this attribute to validate the amount of data read from the stream matches
        # the content length, so we need to reset it as well.
        # https://github.com/boto/botocore/blob/f88e981cb1a6cd0c64bc89da262ab76f9bfa9b7d/botocore/response.py#L164C17-L164C32
        response_body._amount_read = 0


@skip_if_trace_disabled
def _patched_converse(original, self, *args, **kwargs):
    with mlflow.start_span(
        name=f"{_BEDROCK_SPAN_PREFIX}{original.__name__}",
        span_type=SpanType.CHAT_MODEL,
    ) as span:
        # NB: Bedrock client doesn't accept any positional arguments
        span.set_inputs(kwargs)
        _set_tool_attributes(span, kwargs)

        result = None
        try:
            result = original(self, *args, **kwargs)
            span.set_outputs(result)
        finally:
            _set_chat_messages_attributes(span, kwargs.get("messages", []), result)
        return result


@skip_if_trace_disabled
def _patched_converse_stream(original, self, *args, **kwargs):
    # NB: Do not use fluent API to create a span for streaming response. If we do so,
    # the span context will remain active until the stream is fully exhausted, which
    # can lead to super hard-to-debug issues.
    span = start_span_no_context(
        name=f"{_BEDROCK_SPAN_PREFIX}{original.__name__}",
        span_type=SpanType.CHAT_MODEL,
        inputs=kwargs,
    )
    _set_tool_attributes(span, kwargs)

    result = original(self, *args, **kwargs)

    if span:
        result["stream"] = ConverseStreamWrapper(
            stream=result["stream"],
            span=span,
            inputs=kwargs,
        )

    return result


def _set_chat_messages_attributes(
    span, messages: list[dict[str, Any]], response: Optional[dict[str, Any]]
):
    """
    Extract standard chat span attributes for the Bedrock Converse API call.

    NB: We only support standard attribute extraction for the Converse API, because
    the InvokeModel API exposes the raw API spec from each LLM provider, hence
    maintaining the compatibility for all providers is significantly cumbersome.
    """
    try:
        messages = [*messages]  # shallow copy to avoid appending to the original list
        if response:
            messages.append(response["output"]["message"])
        messages = [convert_message_to_mlflow_chat(msg) for msg in messages]
        set_span_chat_messages(span, messages)
    except Exception as e:
        _logger.debug(f"Failed to set messages for {span}. Error: {e}")


def _set_tool_attributes(span, kwargs):
    """Extract tool attributes for the Bedrock Converse API call."""
    if tool_config := kwargs.get("toolConfig"):
        try:
            tools = [convert_tool_to_mlflow_chat_tool(tool) for tool in tool_config["tools"]]
            set_span_chat_tools(span, tools)
        except Exception as e:
            _logger.debug(f"Failed to set tools for {span}. Error: {e}")



import json
from typing import Optional

from opentelemetry.context import Context
from opentelemetry.sdk.trace import ReadableSpan as OTelReadableSpan
from opentelemetry.sdk.trace import Span as OTelSpan
from opentelemetry.sdk.trace.export import BatchSpanProcessor, SpanExporter

from mlflow.entities.trace_info import TraceInfo
from mlflow.entities.trace_location import TraceLocation
from mlflow.entities.trace_state import TraceState
from mlflow.tracing.constant import TRACE_SCHEMA_VERSION, TRACE_SCHEMA_VERSION_KEY, SpanAttributeKey
from mlflow.tracing.trace_manager import InMemoryTraceManager
from mlflow.tracing.utils import generate_trace_id_v3


class OtelSpanProcessor(BatchSpanProcessor):
    """
    SpanProcessor implementation to export MLflow traces to a OpenTelemetry collector.

    Extending OpenTelemetry BatchSpanProcessor to add some custom hooks to be executed when a span
    is started or ended (before exporting).
    """

    def __init__(self, span_exporter: SpanExporter):
        super().__init__(span_exporter)
        # In opentelemetry-sdk 1.34.0, the `span_exporter` field was removed from the
        # `BatchSpanProcessor` class.
        # https://github.com/open-telemetry/opentelemetry-python/issues/4616
        #
        # The `span_exporter` field was restored as a property in 1.34.1
        # https://github.com/open-telemetry/opentelemetry-python/pull/4621
        #
        # We use a try-except block to maintain compatibility with both versions.
        try:
            self.span_exporter = span_exporter
        except AttributeError:
            pass
        self._trace_manager = InMemoryTraceManager.get_instance()

    def on_start(self, span: OTelSpan, parent_context: Optional[Context] = None):
        """
        Handle the start of a span. This method is called when an OpenTelemetry span is started.

        Args:
            span: An OpenTelemetry Span object that is started.
            parent_context: The context of the span. Note that this is only passed when the context
                object is explicitly specified to OpenTelemetry start_span call. If the parent
                span is obtained from the global context, it won't be passed here so we should not
                rely on it.
        """
        trace_id = generate_trace_id_v3(span)
        trace_info = TraceInfo(
            trace_id=trace_id,
            trace_location=TraceLocation.from_experiment_id(None),
            request_time=span.start_time // 1_000_000,  # nanosecond to millisecond
            execution_duration=None,
            state=TraceState.IN_PROGRESS,
            trace_metadata={TRACE_SCHEMA_VERSION_KEY: str(TRACE_SCHEMA_VERSION)},
            tags={},
        )
        span.set_attribute(SpanAttributeKey.REQUEST_ID, json.dumps(trace_id))

        self._trace_manager.register_trace(span.context.trace_id, trace_info)

        super().on_start(span, parent_context)

    def on_end(self, span: OTelReadableSpan):
        # Pops the trace entry from the in-memory trace manager to avoid memory leak
        if span._parent is None:
            self._trace_manager.pop_trace(span.context.trace_id)

        super().on_end(span)



"""
The ``mlflow.tracking`` module provides a Python CRUD interface to MLflow experiments
and runs. This is a lower level API that directly translates to MLflow
`REST API <../rest-api.html>`_ calls.
For a higher level API for managing an "active run", use the :py:mod:`mlflow` module.
"""

# Minimum APIs required for core tracing functionality of mlflow-tracing package.
from mlflow.tracking._tracking_service.utils import (
    _get_artifact_repo,
    _get_store,
    get_tracking_uri,
    is_tracking_uri_set,
    set_tracking_uri,
)
from mlflow.version import IS_TRACING_SDK_ONLY

__all__ = [
    "get_tracking_uri",
    "set_tracking_uri",
    "is_tracking_uri_set",
    "_get_artifact_repo",
    "_get_store",
]

# Importing the following APIs only if mlflow or mlflow-skinny is installed.
if not IS_TRACING_SDK_ONLY:
    from mlflow.tracking._model_registry.utils import (
        get_registry_uri,
        set_registry_uri,
    )
    from mlflow.tracking._tracking_service.utils import _get_artifact_repo
    from mlflow.tracking.client import MlflowClient

    __all__ += [
        "get_registry_uri",
        "set_registry_uri",
        "MlflowClient",
    ]



import json
import logging
from typing import TYPE_CHECKING, Any, Optional, Union

from mlflow.entities import Assessment, Trace
from mlflow.exceptions import MlflowException
from mlflow.genai.evaluation.constant import (
    AgentEvaluationReserverKey,
)
from mlflow.genai.scorers import Scorer
from mlflow.models import EvaluationMetric

try:
    # `pandas` is not required for `mlflow-skinny`.
    import pandas as pd
except ImportError:
    pass

if TYPE_CHECKING:
    from mlflow.genai.datasets import EvaluationDataset

    try:
        import pyspark.sql.dataframe

        EvaluationDatasetTypes = Union[
            pd.DataFrame, pyspark.sql.dataframe.DataFrame, list[dict], EvaluationDataset
        ]
    except ImportError:
        EvaluationDatasetTypes = Union[pd.DataFrame, list[dict], EvaluationDataset]


_logger = logging.getLogger(__name__)


def _convert_eval_set_to_df(data: "EvaluationDatasetTypes") -> "pd.DataFrame":
    """
    Takes in a dataset in the format that `mlflow.genai.evaluate()` expects and
    converts it into a pandas DataFrame.
    """
    if isinstance(data, list):
        # validate that every item in the list is a dict and has inputs as key
        for item in data:
            if not isinstance(item, dict):
                raise MlflowException.invalid_parameter_value(
                    "Every item in the list must be a dictionary."
                )
        df = pd.DataFrame(data)
    elif isinstance(data, pd.DataFrame):
        # Data is already a pd DataFrame, just copy it
        df = data.copy()
    else:
        try:
            from mlflow.utils.spark_utils import get_spark_dataframe_type

            if isinstance(data, get_spark_dataframe_type()):
                df = _deserialize_inputs_and_expectations_column(data.toPandas())
            else:
                raise MlflowException.invalid_parameter_value(
                    "Invalid type for parameter `data`. Expected a list of dictionaries, "
                    f"a pandas DataFrame, or a Spark DataFrame. Got: {type(data)}"
                )
        except ImportError:
            raise ImportError(
                "The `pyspark` package is required to use mlflow.genai.evaluate() "
                "Please install it with `pip install pyspark`."
            )

    if len(df) == 0:
        raise MlflowException.invalid_parameter_value(
            "The dataset is empty. Please provide a non-empty dataset."
        )

    if not any(col in df.columns for col in ("trace", "inputs")):
        raise MlflowException.invalid_parameter_value(
            "Either `inputs` or `trace` column is required in the dataset. Please provide inputs "
            "for every datapoint or provide a trace."
        )

    return df


def _convert_to_legacy_eval_set(data: "EvaluationDatasetTypes") -> "pd.DataFrame":
    """
    Takes in a dataset in the format that mlflow.genai.evaluate() expects and converts it into
    to the current eval-set schema that Agent Evaluation takes in. The transformed schema should
    be accepted by mlflow.evaluate().
    The expected schema can be found at:
    https://docs.databricks.com/aws/en/generative-ai/agent-evaluation/evaluation-schema

    NB: The harness secretly support 'expectations' column as well. It accepts a dictionary of
        expectations, which is same as the schema that mlflow.genai.evaluate() expects.
        Therefore, we can simply pass through expectations column.
    """
    column_mapping = {
        "inputs": "request",
        "outputs": "response",
    }

    df = _convert_eval_set_to_df(data)

    return (
        df.rename(columns=column_mapping)
        .pipe(_deserialize_trace_column_if_needed)
        .pipe(_extract_request_from_trace)
        .pipe(_extract_expectations_from_trace)
    )


def _deserialize_inputs_and_expectations_column(df: "pd.DataFrame") -> "pd.DataFrame":
    """
    Deserialize the `inputs` and `expectations` string columns from the dataframe.

    When managed datasets are read as Spark DataFrames, the `inputs` and `expectations` columns
    are loaded as string columns of JSON strings. This function deserializes these columns into
    dictionaries expected by mlflow.genai.evaluate().
    """
    target_columns = ["inputs", "expectations"]
    for col in target_columns:
        if col not in df.columns or not isinstance(df[col][0], str):
            continue

        try:
            df[col] = df[col].apply(json.loads)
        except json.JSONDecodeError as e:
            if col == "inputs":
                msg = (
                    "The `inputs` column must be a valid JSON string of field names and values. "
                    "For example, `{'question': 'What is the capital of France?'}`"
                )
            else:
                msg = (
                    "The `expectations` column must be a valid JSON string of assessment names and "
                    "values. For example, `{'expected_facts': ['fact1', 'fact2']}`"
                )
            raise MlflowException.invalid_parameter_value(
                f"Failed to parse `{col}` column. Error: {e}\nHint: {msg}"
            )

    return df


def _deserialize_trace_column_if_needed(df: "pd.DataFrame") -> "pd.DataFrame":
    """
    Deserialize the `trace` column from the dataframe if it is a string.

    Since MLflow 3.2.0, mlflow.search_traces() returns a pandas DataFrame with a `trace`
    column that is a trace json representation rather than the Trace object itself. This
    function deserializes the `trace` column into a Trace object.
    """
    if "trace" in df.columns:
        df["trace"] = df["trace"].apply(lambda t: Trace.from_json(t) if isinstance(t, str) else t)
    return df


def _extract_request_from_trace(df: "pd.DataFrame") -> "pd.DataFrame":
    """
    Add `request` columns to the dataframe if it is not already present.
    This is for compatibility with mlflow.evaluate() that requires `request` column.
    """
    if "trace" not in df.columns:
        return df

    if "request" not in df.columns:
        df["request"] = df["trace"].apply(lambda trace: json.loads(trace.data.request))
    return df


def _extract_expectations_from_trace(df: "pd.DataFrame") -> "pd.DataFrame":
    """
    Add `expectations` columns to the dataframe from assessments
    stored in the traces, if the "expectations" column is not already present.
    """
    if "trace" not in df.columns:
        return df

    expectations_column = []
    for trace in df["trace"]:
        expectations = {}
        for assessment in trace.info.assessments or []:
            if assessment.expectation is not None:
                expectations[assessment.name] = assessment.expectation.value
        expectations_column.append(expectations)
    # If no trace has assessments, not add the column
    if all(len(expectations) == 0 for expectations in expectations_column):
        return df

    df["expectations"] = expectations_column
    return df


def _convert_scorer_to_legacy_metric(scorer: Scorer) -> EvaluationMetric:
    """
    Takes in a Scorer object and converts it into a legacy MLflow 2.x
    Metric object.
    """
    try:
        from databricks.agents.evals import metric
    except ImportError:
        raise ImportError(
            "The `databricks-agents` package is required to use mlflow.genai.evaluate() "
            "Please install it with `pip install databricks-agents`."
        )

    from mlflow.types.llm import ChatCompletionRequest

    def eval_fn(
        request_id: str,
        request: Union[ChatCompletionRequest, str],
        response: Optional[Any],
        expected_response: Optional[Any],
        trace: Optional[Trace],
        guidelines: Optional[Union[list[str], dict[str, list[str]]]],
        expected_facts: Optional[list[str]],
        expected_retrieved_context: Optional[list[dict[str, str]]],
        custom_expected: Optional[dict[str, Any]],
        **kwargs,
    ) -> Union[int, float, bool, str, Assessment, list[Assessment]]:
        # Condense all expectations into a single dict
        expectations = {}
        if expected_response is not None:
            expectations[AgentEvaluationReserverKey.EXPECTED_RESPONSE] = expected_response
        if expected_facts is not None:
            expectations[AgentEvaluationReserverKey.EXPECTED_FACTS] = expected_facts
        if expected_retrieved_context is not None:
            expectations[AgentEvaluationReserverKey.EXPECTED_RETRIEVED_CONTEXT] = (
                expected_retrieved_context
            )
        if guidelines is not None:
            expectations[AgentEvaluationReserverKey.GUIDELINES] = guidelines
        if custom_expected is not None:
            expectations.update(custom_expected)

        merged = {
            "inputs": request,
            "outputs": response,
            "expectations": expectations,
            "trace": trace,
        }
        return scorer.run(**merged)

    return metric(
        eval_fn=eval_fn,
        name=scorer.name,
    )



"""
Genesis-Flow Security Validation

This module provides comprehensive input validation to prevent
injection attacks, path traversal, and other security vulnerabilities.
"""

import os
import re
import logging
from pathlib import Path
from typing import Union, Optional
from urllib.parse import urlparse

logger = logging.getLogger(__name__)

class SecurityValidationError(Exception):
    """Exception raised for security validation failures."""
    pass

class InputValidator:
    """
    Comprehensive input validation for Genesis-Flow.
    """
    
    # Safe characters for different input types
    SAFE_NAME_PATTERN = re.compile(r'^[a-zA-Z0-9_\-\.]+$')
    SAFE_TAG_KEY_PATTERN = re.compile(r'^[a-zA-Z0-9_\-\.\/]+$')
    SAFE_TAG_VALUE_PATTERN = re.compile(r'^[a-zA-Z0-9_\-\.\s\/\:\@]+$')
    
    # Maximum lengths to prevent DoS
    MAX_NAME_LENGTH = 256
    MAX_TAG_KEY_LENGTH = 250
    MAX_TAG_VALUE_LENGTH = 5000
    MAX_DESCRIPTION_LENGTH = 10000
    MAX_ARTIFACT_PATH_LENGTH = 1000
    
    @staticmethod
    def validate_experiment_name(name: str) -> str:
        """
        Validate experiment name against injection attacks.
        
        Args:
            name: Experiment name to validate
            
        Returns:
            Validated name
            
        Raises:
            SecurityValidationError: If name is invalid
        """
        if not name or not isinstance(name, str):
            raise SecurityValidationError("Experiment name must be a non-empty string")
        
        if len(name) > InputValidator.MAX_NAME_LENGTH:
            raise SecurityValidationError(
                f"Experiment name too long (max {InputValidator.MAX_NAME_LENGTH} chars)"
            )
        
        # Check for dangerous characters
        if not InputValidator.SAFE_NAME_PATTERN.match(name):
            raise SecurityValidationError(
                "Experiment name contains invalid characters. "
                "Only alphanumeric, dash, underscore, and dot allowed."
            )
        
        # Prevent directory traversal
        if '..' in name or '/' in name or '\\' in name:
            raise SecurityValidationError("Experiment name cannot contain path traversal sequences")
        
        logger.debug(f"Validated experiment name: {name}")
        return name
    
    @staticmethod
    def validate_run_name(name: Optional[str]) -> Optional[str]:
        """
        Validate run name.
        
        Args:
            name: Run name to validate (can be None)
            
        Returns:
            Validated name or None
        """
        if name is None:
            return None
            
        if not isinstance(name, str):
            raise SecurityValidationError("Run name must be a string")
        
        if len(name) > InputValidator.MAX_NAME_LENGTH:
            raise SecurityValidationError(
                f"Run name too long (max {InputValidator.MAX_NAME_LENGTH} chars)"
            )
        
        # Allow more flexible run names but still prevent injection
        if '..' in name or '\x00' in name:
            raise SecurityValidationError("Run name contains invalid sequences")
        
        return name
    
    @staticmethod
    def validate_tag_key(key: str) -> str:
        """
        Validate tag key.
        
        Args:
            key: Tag key to validate
            
        Returns:
            Validated key
        """
        if not key or not isinstance(key, str):
            raise SecurityValidationError("Tag key must be a non-empty string")
        
        if len(key) > InputValidator.MAX_TAG_KEY_LENGTH:
            raise SecurityValidationError(
                f"Tag key too long (max {InputValidator.MAX_TAG_KEY_LENGTH} chars)"
            )
        
        if not InputValidator.SAFE_TAG_KEY_PATTERN.match(key):
            raise SecurityValidationError(
                "Tag key contains invalid characters"
            )
        
        return key
    
    @staticmethod
    def validate_tag_value(value: str) -> str:
        """
        Validate tag value.
        
        Args:
            value: Tag value to validate
            
        Returns:
            Validated value
        """
        if not isinstance(value, str):
            raise SecurityValidationError("Tag value must be a string")
        
        if len(value) > InputValidator.MAX_TAG_VALUE_LENGTH:
            raise SecurityValidationError(
                f"Tag value too long (max {InputValidator.MAX_TAG_VALUE_LENGTH} chars)"
            )
        
        # Check for control characters and potential injection
        if '\x00' in value or '\r' in value:
            raise SecurityValidationError("Tag value contains invalid control characters")
        
        return value
    
    @staticmethod
    def validate_artifact_path(path: str, base_path: Optional[str] = None) -> str:
        """
        Validate artifact path to prevent path traversal attacks.
        
        Args:
            path: Artifact path to validate
            base_path: Optional base path for additional validation
            
        Returns:
            Validated path
            
        Raises:
            SecurityValidationError: If path is unsafe
        """
        if not path or not isinstance(path, str):
            raise SecurityValidationError("Artifact path must be a non-empty string")
        
        if len(path) > InputValidator.MAX_ARTIFACT_PATH_LENGTH:
            raise SecurityValidationError(
                f"Artifact path too long (max {InputValidator.MAX_ARTIFACT_PATH_LENGTH} chars)"
            )
        
        # Normalize the path to resolve any relative components
        normalized_path = os.path.normpath(path)
        
        # Check for path traversal attempts
        if '..' in normalized_path or normalized_path.startswith('/'):
            raise SecurityValidationError("Artifact path contains path traversal sequences")
        
        # Additional check if base path provided
        if base_path:
            base = Path(base_path).resolve()
            try:
                target = (base / normalized_path).resolve()
                # Ensure target is within base directory
                target.relative_to(base)
            except ValueError:
                raise SecurityValidationError("Artifact path escapes base directory")
        
        logger.debug(f"Validated artifact path: {normalized_path}")
        return normalized_path
    
    @staticmethod
    def validate_metric_key(key: str) -> str:
        """
        Validate metric key.
        
        Args:
            key: Metric key to validate
            
        Returns:
            Validated key
        """
        if not key or not isinstance(key, str):
            raise SecurityValidationError("Metric key must be a non-empty string")
        
        if len(key) > InputValidator.MAX_TAG_KEY_LENGTH:
            raise SecurityValidationError(
                f"Metric key too long (max {InputValidator.MAX_TAG_KEY_LENGTH} chars)"
            )
        
        # Metric keys should be more restrictive
        if not re.match(r'^[a-zA-Z0-9_\-\.]+$', key):
            raise SecurityValidationError(
                "Metric key contains invalid characters. "
                "Only alphanumeric, dash, underscore, and dot allowed."
            )
        
        return key
    
    @staticmethod
    def validate_param_key(key: str) -> str:
        """
        Validate parameter key.
        
        Args:
            key: Parameter key to validate
            
        Returns:
            Validated key
        """
        if not key or not isinstance(key, str):
            raise SecurityValidationError("Parameter key must be a non-empty string")
        
        if len(key) > InputValidator.MAX_TAG_KEY_LENGTH:
            raise SecurityValidationError(
                f"Parameter key too long (max {InputValidator.MAX_TAG_KEY_LENGTH} chars)"
            )
        
        # Parameter keys should be restrictive
        if not re.match(r'^[a-zA-Z0-9_\-\.]+$', key):
            raise SecurityValidationError(
                "Parameter key contains invalid characters. "
                "Only alphanumeric, dash, underscore, and dot allowed."
            )
        
        return key
    
    @staticmethod
    def validate_param_value(value: Union[str, int, float]) -> str:
        """
        Validate parameter value.
        
        Args:
            value: Parameter value to validate
            
        Returns:
            Validated value as string
        """
        if value is None:
            raise SecurityValidationError("Parameter value cannot be None")
        
        # Convert to string if needed
        str_value = str(value)
        
        if len(str_value) > InputValidator.MAX_TAG_VALUE_LENGTH:
            raise SecurityValidationError(
                f"Parameter value too long (max {InputValidator.MAX_TAG_VALUE_LENGTH} chars)"
            )
        
        # Check for control characters
        if '\x00' in str_value or '\r' in str_value:
            raise SecurityValidationError("Parameter value contains invalid control characters")
        
        return str_value
    
    @staticmethod
    def validate_model_name(name: str) -> str:
        """
        Validate registered model name.
        
        Args:
            name: Model name to validate
            
        Returns:
            Validated name
        """
        if not name or not isinstance(name, str):
            raise SecurityValidationError("Model name must be a non-empty string")
        
        if len(name) > InputValidator.MAX_NAME_LENGTH:
            raise SecurityValidationError(
                f"Model name too long (max {InputValidator.MAX_NAME_LENGTH} chars)"
            )
        
        # Model names should be safe for file systems and URLs
        if not re.match(r'^[a-zA-Z0-9_\-\.]+$', name):
            raise SecurityValidationError(
                "Model name contains invalid characters. "
                "Only alphanumeric, dash, underscore, and dot allowed."
            )
        
        # Prevent directory traversal
        if '..' in name:
            raise SecurityValidationError("Model name cannot contain '..'")
        
        return name
    
    @staticmethod
    def validate_uri(uri: str, allowed_schemes: Optional[set] = None) -> str:
        """
        Validate URI for safety.
        
        Args:
            uri: URI to validate
            allowed_schemes: Set of allowed URI schemes
            
        Returns:
            Validated URI
        """
        if not uri or not isinstance(uri, str):
            raise SecurityValidationError("URI must be a non-empty string")
        
        if allowed_schemes is None:
            allowed_schemes = {'http', 'https', 'file', 's3', 'gs', 'azure'}
        
        try:
            parsed = urlparse(uri)
        except Exception as e:
            raise SecurityValidationError(f"Invalid URI format: {e}")
        
        if parsed.scheme and parsed.scheme not in allowed_schemes:
            raise SecurityValidationError(
                f"URI scheme '{parsed.scheme}' not allowed. "
                f"Allowed schemes: {sorted(allowed_schemes)}"
            )
        
        # Check for potential SSRF attacks
        if parsed.hostname:
            # Block private/local addresses for HTTP schemes
            if parsed.scheme in ('http', 'https'):
                if parsed.hostname.lower() in ('localhost', '127.0.0.1', '0.0.0.0'):
                    raise SecurityValidationError("URI points to localhost/loopback address")
                
                # Block private IP ranges (basic check)
                if (parsed.hostname.startswith('192.168.') or 
                    parsed.hostname.startswith('10.') or
                    parsed.hostname.startswith('172.')):
                    raise SecurityValidationError("URI points to private IP address")
        
        return uri

def secure_filename(filename: str) -> str:
    """
    Secure a filename by removing/replacing dangerous characters.
    
    Args:
        filename: Original filename
        
    Returns:
        Secured filename
    """
    if not filename:
        raise SecurityValidationError("Filename cannot be empty")
    
    # Remove path components
    filename = os.path.basename(filename)
    
    # Replace dangerous characters
    filename = re.sub(r'[^\w\-_\.]', '_', filename)
    
    # Ensure it doesn't start with dot (hidden file)
    if filename.startswith('.'):
        filename = '_' + filename[1:]
    
    # Ensure reasonable length
    if len(filename) > 255:
        name, ext = os.path.splitext(filename)
        filename = name[:251-len(ext)] + ext
    
    return filename